package ludo

import (
	"server/game"
	"server/msg"
	"time"
)

type RoomInfoWrapper struct {
	*msg.RoomInfo
}

func init() {

}

// 清理解散的房间
func ClearRoomInfoWrapperDisbandRoom() {
	for k := range gameConfig.BattleRoom {
		if gameConfig.BattleRoom[k].RoomStatus == msg.RoomStatus_END {
			delete(gameConfig.BattleRoom, k)
		}
	}
}

func runBattle(room_info *RoomInfoWrapper) {
	// cur_color_key := getCurColorEventKey(room_info)
	// TimerManager.AddEvent(cur_color_key,)
	gameConfig.BattleRoom[room_info.Id] = room_info
	game.Module.AfterFunc(time.Second*time.Duration(3), func() {
		room_info.notify_player_sz()
	})
}

func getBattleRoomById(id int32) *RoomInfoWrapper {
	room_info, exists := gameConfig.BattleRoom[id]
	if !exists {
		return nil
	}
	return room_info
}

// 根据玩家的骰子数进行相关处理
func (room_info *RoomInfoWrapper) processSz(msg_body *msg.NotifyPlayerSzNumber) {
	if msg_body.SzNumber == 6 {
		canMoveList := room_info.getRolesBySzNumber(msg_body.Color, msg_body.SzNumber)
		if len(canMoveList) > 0 {
			room_info.NotifyToAllPlayer(msg_body)
		} else {
			room_info.setNextPlayerRound()
			room_info.notify_player_sz()
		}
	} else {
		notAtHomeRoles := room_info.getRoleNotAtHomeByColor(msg_body.Color)
		if len(notAtHomeRoles) <= 0 { //全部都在家
			room_info.setNextPlayerRound()
			room_info.notify_player_sz()
		} else if len(notAtHomeRoles) == 4 { //全部都从家里出来了
			canMoveList := room_info.getCanMoveRoles(notAtHomeRoles, msg_body.SzNumber)
			if len(canMoveList) > 0 {
				room_info.notify_player_move(canMoveList)
			} else {
				room_info.setNextPlayerRound()
				room_info.notify_player_sz()
			}
		} else {
			if len(notAtHomeRoles) == 1 { //只有一个选择，那么直接让他走
				//如果一个玩家离终点剩余的步骤不等于筛子数，也将会跳到下个人回合
				if room_info.roleIsCanMove(notAtHomeRoles[0].MId, msg_body.SzNumber) {
					room_info.send_role_move(msg_body.Color, notAtHomeRoles[0].MId)
				} else {
					room_info.setNextPlayerRound()
					room_info.notify_player_sz()
				}
			} else {
				canMoveList := room_info.getCanMoveRoles(notAtHomeRoles, msg_body.SzNumber)
				if len(canMoveList) > 0 {
					room_info.notify_player_move(canMoveList)
				} else {
					room_info.setNextPlayerRound()
					room_info.notify_player_sz()
				}
			}
		}
	}
}

// 根据玩家走的步数进行相关处理
func (room_info *RoomInfoWrapper) processMove(color msg.RoleType, szNumber int32, isAddRound bool, isFinish bool) {
	if room_info.IsOverGame() { //宣布游戏结束
		room_info.notify_settlement(color)
	} else {
		if szNumber == 6 || isAddRound || isFinish { //加一个回合

		} else { //下个人回合
			room_info.setNextPlayerRound()
		}
		room_info.notify_player_sz()
	}
}

// 通知玩家结算
func (room_info *RoomInfoWrapper) notify_settlement(color msg.RoleType) {
	message := &msg.NotifySettlement{}
	message.Color = color
	message.FinishColors = room_info.getFinishColors()
	room_info.NotifyToAllPlayer(message)
}

// 通知玩家开始移动角色
func (room_info *RoomInfoWrapper) notify_player_move(list []*msg.RoleData) {
	cur_color := room_info.CurRoundColor
	if len(list) == 0 {
		room_info.send_role_move(cur_color, list[0].MId)
	} else {
		if room_info.containsRobot(cur_color) {
			//如果是机器人，思考一秒然后执行移动角色
			room_info.delayActionMoveRobot(1000, cur_color, list)
		}
	}

	message := &msg.NotifyPlayerOpt{
		Color:        room_info.CurRoundColor,
		Opt:          msg.OptType_SELECT_ROLE,
		CanMoveRoles: list,
	}
	room_info.NotifyToAllPlayer(message)
}

// 通知玩家状态
func (room_info *RoomInfoWrapper) notify_player_status(color msg.RoleType, status msg.PlayerStatus, colors []*msg.ColorData) {
	message := &msg.NotifyPlayerStatus{
		Color:  color,
		Status: status,
		Colors: colors,
	}
	room_info.NotifyToAllPlayer(message)
}

// 通知玩家要骰子
func (room_info *RoomInfoWrapper) notify_player_sz() {
	cur_color := room_info.CurRoundColor
	message := &msg.NotifyPlayerOpt{
		Color: cur_color,
		Opt:   msg.OptType_ZHI_SHAI_ZI,
	}
	room_info.NotifyToAllPlayer(message)
}

// 获取房间ID
func (room_info *RoomInfoWrapper) GetRoomId() int32 {
	return room_info.Id
}

// 房间里通知所有玩家
func (room_info *RoomInfoWrapper) NotifyToAllPlayer(message interface{}) {
	for i := range room_info.Colors {
		user_id := room_info.Colors[i].MId
		user_agent := getAgentByUserId(user_id)
		if user_agent != nil {
			user_agent.WriteMsg(message)
		}
	}
}

// 给前端展示一段时间后，再操作
func (room_info *RoomInfoWrapper) delayActionNextRound(td int32, color msg.RoleType, szNumber int32, isAddRound bool, isFinish bool) {
	dtn := time.Duration(td) * time.Microsecond
	game.Module.Skeleton.AfterFunc(dtn, func() {
		room_info.processMove(color, szNumber, isAddRound, isFinish)
	})
}

// 给前端展示一段时间后，再操作
func (room_info *RoomInfoWrapper) delayActionSz(td int, msg_body *msg.NotifyPlayerSzNumber) {
	dtn := time.Duration(td) * time.Microsecond
	game.Module.Skeleton.AfterFunc(dtn, func() {
		room_info.processSz(msg_body)
	})
}

// 延迟后，移动机器人
func (room_info *RoomInfoWrapper) delayActionMoveRobot(td int, color msg.RoleType, list []*msg.RoleData) {
	dtn := time.Duration(td) * time.Microsecond
	game.Module.Skeleton.AfterFunc(dtn, func() {
		role := getRandomItem(list)
		room_info.send_role_move(color, role.MId)
	})
}
